unit dmGeneral;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DBTables, DspQuery, AdvDspQuery, RxQuery;

type
  Tdm_General = class(TDataModule)
    tbl_Orders: TAdvDspQuery;
    tbl_OrderLines: TDspQuery;
    ds_Orders: TDataSource;
    tbl_OrdersIdOrder: TIntegerField;
    tbl_OrdersCustomer: TIntegerField;
    tbl_OrdersTotal: TFloatField;
    tbl_OrderLinesIdOrderLine: TIntegerField;
    tbl_OrderLinesIdOrder: TIntegerField;
    tbl_OrderLinesPrice: TFloatField;
    tbl_OrderLinesDiscount: TFloatField;
    tbl_OrderLinesAmount: TFloatField;
    tbl_OrderLinesTotal: TFloatField;
    tbl_OrderLinesNetPrice: TFloatField;
    tbl_OrdersIdCurrency: TIntegerField;
    tbl_OrdersDiscount: TFloatField;
    tbl_OrderLinesIdCurrency: TIntegerField;
    tbl_OrdersCustomerName: TStringField;
    dsRO: TQuery;
    tbl_OrderLinesItem: TStringField;
    procedure dm_GeneralCreate(Sender: TObject);
    procedure tbl_OrdersCustomerValidate(Sender: TField);
    procedure tbl_OrderLinesNetPriceCalc(DataSet: TDataSet);
    procedure tbl_OrderLinesAfterPost(DataSet: TDataSet);
    procedure tbl_OrderLinesBeforeDelete(DataSet: TDataSet);
    procedure tbl_OrdersBeforeDelete(DataSet: TDataSet);
    procedure tbl_OrderLinesCalcFields(DataSet: TDataSet);
    procedure tbl_OrderLinesVirtualFieldsChange(DataSet: TDataSet;
      Field: TField);
    procedure tbl_OrderLinesNewRecord(DataSet: TDataSet);
    procedure tbl_OrdersNewRecord(DataSet: TDataSet);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  dm_General: Tdm_General;

implementation

{$R *.DFM}

procedure Tdm_General.dm_GeneralCreate(Sender: TObject);
begin
	tbl_Orders.Open;
  { Add custom CalcFields }
  // Here to calculate the NetPrice
  tbl_OrderLines.AddCalcEvent('Price;Discount', tbl_OrderLinesNetPriceCalc, False);
  // Here to calculate the Total for the line
  tbl_OrderLines.AddCalcEvent('NetPrice;Amount', tbl_OrderLinesCalcFields, False);
  tbl_OrderLines.Open;
end;

procedure Tdm_General.tbl_OrdersCustomerValidate(Sender: TField);
begin
	{ Search for the discount }
  with dsRO do begin
  	Close;
    SQL.Clear;
    SQL.Add('SELECT Discount FROM Customers WHERE IdCustomer = ' + Sender.AsString);
    Open;
  end;

  if not dsRO.Eof then
  	tbl_OrdersDiscount.Value := dsRO['Discount']
  else
  	raise Exception.Create('Customer not found');
end;

procedure Tdm_General.tbl_OrderLinesNetPriceCalc(DataSet: TDataSet);
begin
	tbl_OrderLinesNetPrice.Value := tbl_OrderLinesPrice.Value * (1-tbl_OrderLinesDiscount.Value / 100);
end;

procedure Tdm_General.tbl_OrderLinesAfterPost(DataSet: TDataSet);
begin
	{ Increase the total of the order }
  if tbl_OrderLines.DistinctValue('Total') then begin // If the total changes
    tbl_Orders.DisableControls; {We must disable because on Edit, all the controls
    	asociated to the parent table get refreshed and then the oldvalues losts }
    try
      tbl_Orders.Edit;
      tbl_OrdersTotal.Value := tbl_OrdersTotal.Value +
        (tbl_OrderLinesTotal.Value - tbl_OrderLines.OldValues['Total']);
      tbl_Orders.SafeSave; // Post without risk
    finally
    	tbl_Orders.EnableControls;
    end;
  end;
end;

procedure Tdm_General.tbl_OrderLinesBeforeDelete(DataSet: TDataSet);
begin
	{ If not deleting the parent }
	if not tbl_Orders.Deleting then begin
  	tbl_Orders.Edit;
  	tbl_OrdersTotal.Value := tbl_OrdersTotal.Value - tbl_OrderLinesTotal.Value;
    tbl_Orders.SafeSave;
  end;
end;

procedure Tdm_General.tbl_OrdersBeforeDelete(DataSet: TDataSet);
begin
	{ We must delete the order lines asociated }
  tbl_OrderLines.First;
  while not tbl_OrderLines.EOF do begin
  	tbl_OrderLines.Delete;
  end;
end;

procedure Tdm_General.tbl_OrderLinesCalcFields(DataSet: TDataSet);
var
	Total: Currency;
begin
	{ If we are in WriteMode calc the total }
  if (DataSet.State in [dsEdit, dsInsert]) then begin
	  Total := tbl_OrderLinesNetPrice.Value * tbl_OrderLinesAmount.Value;
    Total := Total * (1 - tbl_OrdersDiscount.Value);
    tbl_OrderLinesTotal.Value := Total;
  end;
end;

procedure Tdm_General.tbl_OrderLinesVirtualFieldsChange(DataSet: TDataSet;
  Field: TField);
begin
	{ We must recalculate the discount }
  if Field = tbl_OrderLinesNetPrice then begin
  	// Recalculamos el descuento
  	tbl_OrderLinesDiscount.Value := - (100 * (tbl_OrderLinesNetPrice.Value -
    	tbl_OrderLinesPrice.Value) / tbl_OrderLinesPrice.Value);
  end;
end;

procedure Tdm_General.tbl_OrderLinesNewRecord(DataSet: TDataSet);
begin
	tbl_OrderLinesDiscount.Value := 0;
end;

procedure Tdm_General.tbl_OrdersNewRecord(DataSet: TDataSet);
begin
	tbl_OrdersTotal.Value := 0;
end;

end.
